@@ -243,6 +243,32 @@ dump_gdk_tree (GdkWindow *window)
 }
 #endif
 
+static CORBA_char *
+bonobo_control_frame_get_remote_window_id (BonoboControlFrame *frame,
+					   CORBA_Environment  *ev)
+{
+#ifdef HAVE_GTK_MULTIHEAD
+	CORBA_char  *retval;
+	char        *cookie;
+	int          screen;
+
+	screen = gdk_screen_get_number (
+			gtk_widget_get_screen (frame->priv->socket));
+
+	cookie = g_strdup_printf ("screen=%d", screen);
+
+	retval = Bonobo_Control_getWindowId (
+				frame->priv->control, cookie, ev);
+
+	g_free (cookie);
+
+	return retval;
+#else
+	return Bonobo_Control_getWindowId (
+				frame->priv->control, "", ev);
+#endif /* HAVE_GTK_MULTIHEAD */
+}
+
 void
 bonobo_control_frame_get_remote_window (BonoboControlFrame *frame,
 					CORBA_Environment  *opt_ev)
@@ -270,15 +296,14 @@ bonobo_control_frame_get_remote_window (BonoboControlFrame *frame,
 		ev = opt_ev;
 
 	/* Introduce ourselves to the Control. */
-	id = Bonobo_Control_getWindowId (frame->priv->control, "", ev);
-
+	id = bonobo_control_frame_get_remote_window_id (frame, ev);
 	if (BONOBO_EX (ev)) {
 		dprintf ("getWindowId exception\n");
 		bonobo_object_check_env (BONOBO_OBJECT (frame),
 					 frame->priv->control, ev);
 
 	} else {
-		guint32 xid;
+		GdkNativeWindow xid;
 		BonoboPlug *plug = NULL;
 
 		xid = bonobo_control_x11_from_window_id (id);
@@ -303,7 +328,7 @@ bonobo_control_frame_get_remote_window (BonoboControlFrame *frame,
 		} else if (!plug && frame->priv->inproc_control) 
 			g_warning ("ARGH - different serious ORB screwup");
 
-		gtk_socket_add_id (GTK_SOCKET (frame->priv->socket), xid);
+		bonobo_socket_add_id (BONOBO_SOCKET (frame->priv->socket), xid);
 	}		
 
 	if (!opt_ev)
@@ -939,7 +964,7 @@ bonobo_control_frame_size_request (BonoboControlFrame *frame,
 		/* We haven't been bound to a control yet, so return "I don't
 		 * care about what I get assigned".
 		 */
-		requisition->width = requisition->height = 0;
+		requisition->width = requisition->height = 1;
 		return;
 	}
 
@@ -956,7 +981,7 @@ bonobo_control_frame_size_request (BonoboControlFrame *frame,
 			BONOBO_OBJECT (frame),
 			(CORBA_Object) frame->priv->control, ev);
 
-		req.width = req.height = 0;
+		req.width = req.height = 1;
 	}
 
 	requisition->width  = req.width;
@@ -1100,6 +1125,9 @@ bonobo_control_frame_get_popup_component (BonoboControlFrame *control_frame,
 	popup_container = Bonobo_Control_getPopupContainer (
 		control_frame->priv->control, ev);
 
+	if (BONOBO_EX (ev))
+		return NULL;
+
 	bonobo_ui_component_set_container (ui_component, popup_container, ev);
 
 	Bonobo_Unknown_unref (popup_container, ev);
