@@ -226,6 +226,7 @@ free_nodedata_tree (BonoboUIXml *tree, BonoboUINode *node, gboolean do_overrides
 		return;
 
 	free_nodedata (tree, bonobo_ui_node_get_data (node), do_overrides);
+	bonobo_ui_node_set_data (node, NULL);
 
 	for (l = bonobo_ui_node_children (node); l;
              l = bonobo_ui_node_next (l))
@@ -237,7 +238,7 @@ node_free (BonoboUIXml *tree, BonoboUINode *node)
 {
 	free_nodedata_tree (tree, node, TRUE);
 	bonobo_ui_node_unlink (node);
-	bonobo_ui_node_free (node);
+	bonobo_ui_node_unref (node);
 }
 
 static void
@@ -245,36 +246,29 @@ do_set_id (BonoboUIXml *tree, BonoboUINode *node, gpointer id)
 {
 	BonoboUIXmlData *data;
 
-	if (!node)
-		return;
-
 	data = bonobo_ui_xml_get_data (tree, node);
-
 	data->id = id;
 
+#ifdef UI_XML_DEBUG
 	/* Do some basic validation here ? */
 	{
 		const char *p, *name;
 		
 		if ((name = bonobo_ui_node_get_attr_by_id (node, name_id))) {
-			/*
-			 *  The consequences of this are so unthinkable
-			 * an assertion is warrented.
-			 */
 			for (p = name; *p; p++)
 				g_assert (*p != '/' && *p != '#');
 		}
 	}
+#endif
 
-	for (node = bonobo_ui_node_children (node); node;
-             node = bonobo_ui_node_next (node))
+	for (node = node->children; node; node = node->next)
 		do_set_id (tree, node, id);
 }
 
 static void
 set_id (BonoboUIXml *tree, BonoboUINode *node, gpointer id)
 {
-	for (; node; node = bonobo_ui_node_next (node))
+	for (; node; node = node->next)
 		do_set_id (tree, node, id);
 }
 
@@ -282,8 +276,8 @@ static void
 dump_internals (BonoboUIXml *tree, BonoboUINode *node)
 {
 	int i;
-	char *txt;
 	BonoboUINode *l;
+	const char *txt;
 	static int indent = -4;
 	BonoboUIXmlData *data = bonobo_ui_xml_get_data (tree, node);
 
@@ -292,10 +286,8 @@ dump_internals (BonoboUIXml *tree, BonoboUINode *node)
 	for (i = 0; i < indent; i++)
 		fprintf (stderr, " ");
 
-	fprintf (stderr, "%10s name=\"%10s\" ", bonobo_ui_node_get_name (node),
-		 (txt = bonobo_ui_node_get_attr (node, "name")) ? txt : "NULL");
-	if (txt)
-		bonobo_ui_node_free_string (txt);
+	fprintf (stderr, "%16s name=\"%10s\" ", bonobo_ui_node_get_name (node),
+		 (txt = bonobo_ui_node_peek_attr (node, "name")) ? txt : "NULL");
 
 	fprintf (stderr, "%d len %d", data->dirty,
 		 g_slist_length (data->overridden));
@@ -391,8 +383,7 @@ override_node_with (BonoboUIXml *tree, BonoboUINode *old, BonoboUINode *new)
 
 	if (override) {
 
-		g_signal_emit (G_OBJECT (tree),
-			       signals [OVERRIDE], 0, new, old);
+		g_signal_emit (tree, signals [OVERRIDE], 0, new, old);
 
 		data->overridden = g_slist_prepend (old_data->overridden, old);
 		prune_overrides_by_id (tree, data, data->id);
@@ -401,12 +392,11 @@ override_node_with (BonoboUIXml *tree, BonoboUINode *old, BonoboUINode *new)
 			data->id = old_data->id;
 
 		data->overridden = old_data->overridden;
-		g_signal_emit (G_OBJECT (tree),
-			       signals [REPLACE_OVERRIDE], 0, new, old);
+		g_signal_emit (tree, signals [REPLACE_OVERRIDE], 0, new, old);
 
 /*		fprintf (stderr, "Replace override of '%s' '%s' with '%s' '%s'",
-			 old->name, bonobo_ui_node_get_attr (old, "name"),
-			 new->name, bonobo_ui_node_get-attr (new, "name"));*/
+			 old->name, bonobo_ui_node_peek_attr (old, "name"),
+			 new->name, bonobo_ui_node_peek_attr (new, "name"));*/
 	}
 
 	old_data->overridden = NULL;
@@ -458,7 +448,7 @@ reinstate_old_node (BonoboUIXml *tree, BonoboUINode *node)
 		data->overridden = NULL;
 
 		/* Fire remove while still in tree */
-		g_signal_emit (G_OBJECT (tree), signals [REMOVE], 0, node);
+		g_signal_emit (tree, signals [REMOVE], 0, node);
 		
 		/* Move children across */
 		bonobo_ui_node_move_children (node, old);
@@ -469,7 +459,7 @@ reinstate_old_node (BonoboUIXml *tree, BonoboUINode *node)
 		/* Mark dirty */
 		bonobo_ui_xml_set_dirty (tree, old);
 
-		g_signal_emit (G_OBJECT (tree), signals [REINSTATE], 0, old);
+		g_signal_emit (tree, signals [REINSTATE], 0, old);
 
 		watch_update (tree, old);
 
@@ -482,17 +472,17 @@ reinstate_old_node (BonoboUIXml *tree, BonoboUINode *node)
 		/* Mark dirty */
 		bonobo_ui_xml_set_dirty (tree, node);
 		
-		g_signal_emit (G_OBJECT (tree), signals [RENAME], 0, node);
+		g_signal_emit (tree, signals [RENAME], 0, node);
 		return;
 	} else {
 		/* Mark parent & up dirty */
 		bonobo_ui_xml_set_dirty (tree, node);
 
-		g_signal_emit (G_OBJECT (tree), signals [REMOVE], 0, node);
+		g_signal_emit (tree, signals [REMOVE], 0, node);
 	}
 
 /*		fprintf (stderr, "destroying node '%s' '%s'\n",
-		node->name, bonobo_ui_node_get_attr (node, "name"));*/
+		node->name, bonobo_ui_node_peek_attr (node, "name"));*/
 			
 	bonobo_ui_node_unlink (node);
 	
@@ -503,163 +493,13 @@ reinstate_old_node (BonoboUIXml *tree, BonoboUINode *node)
 	node_free (tree, node);
 }
 
-static BonoboUINode *
-find_child (BonoboUINode *node, const char *name)
-{
-	BonoboUINode *l, *ret = NULL;
-
-	g_return_val_if_fail (name != NULL, NULL);
-	g_return_val_if_fail (node != NULL, NULL);
-
-	for (l = node->children; l && !ret; l = l->next) {
-		const char *txt;
-
-		if ((txt = bonobo_ui_node_get_attr_by_id (l, name_id))) {
-			if (!strcmp (txt, name))
-				ret = l;
-		}
-
-		if (!ret && bonobo_ui_node_has_name (l, name))
-			ret = l;
-	}
-
-	return ret;
-}
-
-/*
- *  This monumental waste of time chewed 2 hours of my life
- * and was to try and help Eazel not have to change their
- * code at all; These routines worked fine, the compat ones
- * were duff.
- */
-/*
-char *
-bonobo_ui_xml_path_escape (const char *path)
-{
-	char *ret, *dest;
-	int   len = 0;
-	const char *p;
-
-	for (p = path; p && *p; p++) {
-		if (*p == '/')
-			len++;
-		len++;
-	}
-	
-	dest = ret = g_malloc (len + 1);
-
-	for (p = path; p && *p; p++) {
-		if (*p == '/' ||
-		    *p == '\\')
-			*dest++ = '\\';
-		*dest++ = *p;
-	}
-	dest [len] = '\0';
-
-	return ret;
-}
-
-char *
-bonobo_ui_xml_path_unescape (const char *path)
-{
-	char *ret, *dest;
-	const char *p;
-	
-	dest = ret = g_malloc (strlen (path) + 1);
-
-	for (p = path; p && *p; p++) {
-		if (*p == '\\')
-			p++;
-		*dest++ = *p;
-	}
-	*dest = '\0';
-	
-	return ret;
-}
-*/
-
-/*
- * bonobo_ui_xml_path_split:
- * @path: the path to split
- * 
- * This function splits a path on the '/' character
- * there is no escaping in place in the path - thus
- * do not use the '/' character in a node name.
- * 
- * Return value: the split path.
- */
-static char **
-bonobo_ui_xml_path_split (const char *path)
-{
-	return g_strsplit (path, "/", -1);
-/*	GSList *string_list = NULL, *l;
-	char *chunk, **ret;
-	int   i, chunks;
-	const char *p;
-
-	g_return_val_if_fail (path != NULL, NULL);
-
-	chunk = g_malloc (strlen (path) + 2);
-	p = path;
-
-	string_list = g_slist_prepend (string_list, chunk);
-	chunks = 1;
-	for (i = 0; p && *p; i++) {
-
-		if (*p == '\\') {
-			p++;
-			chunk [i] = *p++;
-		} else if (*p == '/') {
-			chunk [i] = '\0';
-			p++;
-			if (*p != '\0') {
-				string_list = g_slist_prepend (
-					string_list, &chunk [i] + 1);
-				chunks++;
-			}
-		} else
-			chunk [i] = *p++;
-	}
-	chunk [i] = '\0';
-	g_assert (i < strlen (path) + 2);
-
-	ret = g_new (char *, chunks + 1);
-	ret [chunks] = NULL;
-
-	l = string_list;
-	for (i = chunks - 1; i >= 0; i--) {
-		ret [i] = l->data;
-		l = bonobo_ui_node_next (l);
-	}
-	g_assert (l == NULL);
-	g_slist_free (string_list);
-	g_assert (ret [0] == chunk);
-
-	fprintf (stderr, "Split path '%s' to:\n", path);
-	for (i = 0; ret [i]; i++)
-		fprintf (stderr, "> %s\n", ret [i]);
-
-		return ret;*/
-}
-
-static void
-bonobo_ui_xml_path_freev (char **split)
-{
-	g_strfreev (split);
-
-/*	if (split)
-		g_free (*split);
-
-		g_free (split);*/
-}
-
 static BonoboUINode *
 xml_get_path (BonoboUIXml *tree, const char *path,
 	      gboolean allow_wild, gboolean *wildcard)
 {
 	BonoboUINode *ret;
-	char   **names;
-	int      i;
+	char   **names, *copy;
+	int      i, j, slashes;
 	
 	g_return_val_if_fail (tree != NULL, NULL);
 	g_return_val_if_fail (!allow_wild || wildcard != NULL, NULL);
@@ -677,27 +517,44 @@ xml_get_path (BonoboUIXml *tree, const char *path,
 	if (path [0] != '/')
 		g_warning ("non-absolute path brokenness '%s'", path);
 
-	names = bonobo_ui_xml_path_split (path);
+	for (i = slashes = 0; path [i]; i++)
+		if (path [i] == '/')
+			slashes++;
+
+	names = g_newa (char *, slashes + 2);
+	names [0] = copy = g_alloca (i + 1);
+
+	for (i = j = 0; path [i]; i++) {
+		if (path [i] == '/') {
+			copy [i] = '\0';
+			names [++j] = &copy [i + 1];
+		} else
+			copy [i] = path [i];
+	}
+	copy [i] = '\0';
+	names [++j] = NULL;
+
+/*	names = bonobo_ui_xml_path_split (path); */
 
 	ret = tree->root;
 	for (i = 0; names && names [i]; i++) {
 		if (names [i] [0] == '\0')
 			continue;
 
-/*		g_warning ("Path element '%s'", names [i]);*/
+/*		g_warning ("Path element '%s'", names [i]); */
 
 		if (allow_wild &&
 		    names [i] [0] == '*' &&
 		    names [i] [1] == '\0')
 			*wildcard = TRUE;
 
-		else if (!(ret = find_child (ret, names [i]))) {
-			bonobo_ui_xml_path_freev (names);
+		else if (!(ret = bonobo_ui_node_get_path_child (ret, names [i]))) {
+/*			bonobo_ui_xml_path_freev (names); */
 			return NULL;
 		}
 	}
 		
-	bonobo_ui_xml_path_freev (names);
+/*	bonobo_ui_xml_path_freev (names); */
 
 /*	DUMP_XML (tree, tree->root, "After clean find path"); */
 
@@ -847,8 +704,8 @@ merge (BonoboUIXml *tree, BonoboUINode *current, BonoboUINode **new)
 			b_name = NULL;
 
 /*			printf ("'%s' '%s' with '%s' '%s'\n",
-				a->name, bonobo_ui_node_get_attr (a, "name"),
-				b->name, bonobo_ui_node_get_attr (b, "name"));*/
+				a->name, bonobo_ui_node_peek_attr (a, "name"),
+				b->name, bonobo_ui_node_peek_attr (b, "name"));*/
 			
 			b_name = bonobo_ui_node_get_attr_by_id (b, name_id);
 
@@ -880,8 +737,8 @@ merge (BonoboUIXml *tree, BonoboUINode *current, BonoboUINode **new)
 		nextb = b->next;
 		
 /*		fprintf (stderr, "Transfering '%s' '%s' into '%s' '%s'\n",
-			 b->name, bonobo_ui_node_get_attr (b, "name"),
-			 current->name, bonobo_ui_node_get_attr (current, "name"));*/
+			 b->name, bonobo_ui_node_peek_attr (b, "name"),
+			 current->name, bonobo_ui_node_peek_attr (current, "name"));*/
 
 		bonobo_ui_node_unlink (b);
 
@@ -931,8 +788,6 @@ bonobo_ui_xml_merge (BonoboUIXml  *tree,
 	if (nodes == NULL)
 		return BONOBO_UI_ERROR_OK;
 
-	set_id (tree, nodes, id);
-
 	current = bonobo_ui_xml_get_path (tree, path);
 	if (!current) {
 		BonoboUINode *l, *next;
@@ -945,13 +800,12 @@ bonobo_ui_xml_merge (BonoboUIXml  *tree,
 		return BONOBO_UI_ERROR_INVALID_PATH;
 	}
 
+	set_id (tree, nodes, id);
+
 #ifdef UI_XML_DEBUG
-	{
-		char *txt;
-		fprintf (stderr, "\n\n\nPATH: '%s' '%s\n", current->name,
-			 (txt = bonobo_ui_node_get_attr (current, "name")));
-		bonobo_ui_node_free_string (txt);
-	}
+	fprintf (stderr, "\n\n\nPATH: '%s' '%s\n",
+		 g_quark_to_string (current->name_id),
+		 bonobo_ui_node_peek_attr (current, "name"));
 #endif
 
 	DUMP_XML (tree, tree->root, "Merging in");
@@ -1011,7 +865,7 @@ bonobo_ui_xml_finalize (GObject *object)
 	
 	if (tree->root) {
 		free_nodedata_tree (tree, tree->root, TRUE);
-		bonobo_ui_node_free (tree->root);
+		bonobo_ui_node_unref (tree->root);
 		tree->root = NULL;
 	}
 
